#### další krok refactoringu ##########################3
## abstraktní třida z které se subclassují jednotlivé akce (nelze dát rovnou do ProposalAction kvůli model_name)
## v potomcích je nutno nadefinovat tyto metody
##   setvars - nastaví instance variables na základe catid, mimo jiné musí nastavit
##      @short_action_name - krátký název akce (dřív byla v dbf teď je to víceméně vlastnost třídy)
##      @attrs - atributy záznamu (spoustu toho má automatické defaulty)
##      @kctref (optionally) - pokud to dává smysl. nastaví se podle toho priorita  
##   perform - udělá akci (na základě nastavených proměnných) a vrátí true pokud se povedla (false pokud ne)
##   attributes - musí vrátit hash s attributes pod kterými bude vytvořen, významy polí
##      action - popis akce (optimálně prohnaný linkify)
##      followup_url - kam se přepojit po úspěšně udělané akci
##     optionally:
##      category - název kategorie akce - pokud se neuvede odvodí se od jména class
##      url - nemusí se uvádět, ale pokud se uvede, bude se "Proveď to" chovat postaru
# Jak refaktororovat existujici ProposalActions
# 1. založte novou podtřídu Action s názvem popisujícím akci
# 2. přesuňte inicizalizační část (inicializuj_dle_catid) do setvars (# dekóduj parametry)
# 3. v setvars nastavte @short_action_name
# 4. část `self.assign_attributes` ulož v setvars do @attrs, mohou se vymazat pole
#    short_action, priority, zone_id, category, url
# 5. programuj perform - metoda která akci přímo provede, při použití update vrať result
# 6. v proposal_reason.rb podle kódu akce vyhledat reasons které tuto akci iniciují a nahradit
#    ProposalActoin.update_or_create_by pomocí action specific constructoru
#    

class Action < ProposalAction
  include Rails.application.routes.url_helpers # kvuli helperu často používaným pro followup_url

  def self.model_name
    ProposalAction.model_name
  end
  def short_action2; setvars_if_not_set; @short_action_name  end
  def action_attributes; setvars_if_not_set; @attrs end
  
  def self.update_or_create_by(*pars)
    catid = pars.join('|')
    # gracefully transform oldsyled actions (can be deleted eventually)
    oldstyle_proposal_action = ProposalAction.find_by(catid: catid)
    if oldstyle_proposal_action
      action = oldstyle_proposal_action.becomes!(self)
      action.save!
    else 
      action = self.find_or_create_by(catid: catid)
    end

    default_attrs = { 
      url: "/proposal_actions/#{action.id}/doit",
      category: action.class.to_s
    }
    default_attrs[:priority] = action.priority_from_kctref(@kctref) unless @kctref.nil?
    default_attrs[:zone_id] = @kctref[0..1] unless @kctref.nil?
    action.assign_attributes(default_attrs.merge(action.action_attributes))
    action.save
    action
  end

  def doit
    setvars_if_not_set  # nastav proměnné pro akci
    case perform
      when true  # pokud se akce povedla, tak jí chcem smazat
        destroy unless zavrhnut # pokud je akce zavrhnuta, tak si jí chceme pamatovat
        return true
      when false # pokud se nepovedla, tak jí necháme
        return false
      else       # pokud nevíme, co se stalo, musíme hodit výjimku
        raise "The 'perform' method did not return boolean value, this is a mistake"
    end
  end


  private 

  def setvars_if_not_set
    setvars if @attrs.nil?
    raise "setvars musí nastavit @attrs a @short_action_name" if @attrs.nil?
  end

  # these abstract methods should never be called directly
  def perform
    raise "This abstract method should be redefined by method which performs the action and returns true if it was successfull and false if not"
  end
  def setvars
    raise "This abstract method should be redefined in the child"
  end

end


## v consoli jsou pak delat veci jako
# actions = Action.with_score.select{|a| a.score > 150}
# actions.each {|a| a.doit}